@@ -4,9 +4,21 @@ if (parallel::detectCores()<=2){
   options(restatapi_cores=2)
 }    
 
-
+options(restatapi_verbose=TRUE)
 
 clean_restatapi_cache()
+
+# suppressWarnings(mem_size<-switch(Sys.info()[['sysname']],
+#                                   Windows={tryCatch({as.numeric(gsub("[^0-9]","",system("wmic MemoryChip get Capacity", intern = TRUE)[2]))/1024/1024},error=function(e){0},warning=function(w){0})},
+#                                   Darwin={tryCatch({as.numeric(substring(system('hwprefs memory_size', intern = TRUE), 13))},error=function(e){0},warning=function(w){0})},
+#                                   SunOS={tryCatch({as.numeric(gsub("[^0-9]","",system("prtconf | grep Memory", intern = TRUE,ignore.stderr=TRUE)))},error=function(e){0},warning=function(w){0})},
+#                                   Linux={tryCatch({as.numeric(system("awk '/MemTotal/ {print $2}' /proc/meminfo",intern=TRUE,ignore.stderr=TRUE))/1024},error=function(e){0},warning=function(w){0})}
+# ))
+# 
+# test_that("test mem_size",{
+#   expect_true(mem_size>1)
+# })
+
 context("test of the get_eurostat_toc function")
 t1<-system.time({xml_toc<-get_eurostat_toc(verbose=TRUE)})[3]
 txt_toc<-get_eurostat_toc(mode="txt")
@@ -66,14 +78,14 @@ if (!is.null(dsd)){
 
 
 context("test of the get_eurostat_data function")
-id<-"htec_cis3"
-t1<-system.time({dt1<-get_eurostat_data(id,keep_flags=FALSE,cflags=TRUE)})[3]
+teszt_id0<-"htec_cis3"
+t1<-system.time({dt1<-get_eurostat_data(teszt_id0,keep_flags=FALSE,cflags=TRUE)})[3]
 nc1<-ncol(dt1)
-t2<-system.time({dt2<-get_eurostat_data(id,stringsAsFactors=FALSE)})[3]
+t2<-system.time({dt2<-get_eurostat_data(teszt_id0,stringsAsFactors=FALSE)})[3]
 nc2<-ncol(dt2)
 if (!is.null(dt1)&is.data.frame(dt1)&!is.null(dt2)&is.data.frame(dt2)&!is.null(xml_toc)){
   test_that("test of the get_eurostat_data function", {
-    expect_equal(nrow(dt1),as.numeric(xml_toc$values[xml_toc$code==id]))
+    expect_equal(nrow(dt1),as.numeric(xml_toc$values[xml_toc$code==teszt_id0]))
     expect_equal(nc2+1,nc1)
     expect_true(all(is.numeric(dt1$values)))
     expect_true(all(is.numeric(dt2$values)))
@@ -81,24 +93,25 @@ if (!is.null(dt1)&is.data.frame(dt1)&!is.null(dt2)&is.data.frame(dt2)&!is.null(x
 }
 context("test of the get_eurostat_raw/bulk/data functions")
 if (!is.null(xml_toc)){
-  id<-xml_toc$code[is.na(xml_toc$values)&is.na(xml_toc$lastUpdate)][1]
+  teszt_id0<-xml_toc$code[is.na(xml_toc$values)&is.na(xml_toc$lastUpdate)][1]
   test_that("test of the get_eurostat_raw/bulk/data functions", {
-    expect_message(rt1<-get_eurostat_raw(id,verbose=FALSE))
+    expect_message(rt1<-get_eurostat_raw(teszt_id0,verbose=FALSE))
     expect_equal(rt1,NULL)
-    expect_message(rt2<-get_eurostat_raw(id,check_toc=TRUE,verbose=FALSE))
+    expect_message(rt2<-get_eurostat_raw(teszt_id0,check_toc=TRUE,verbose=FALSE))
     expect_equal(rt2,NULL)
     expect_message(bt1<-get_eurostat_bulk("blabla",check_toc=TRUE,verbose=FALSE))
     expect_equal(bt1,NULL)
-    expect_message(dt3<-get_eurostat_data(id,verbose=FALSE))
+    expect_message(dt3<-get_eurostat_data(teszt_id0,verbose=FALSE))
     expect_equal(dt3,NULL)
   })
 } 
-rt3<-get_eurostat_raw("agr_r_milkpr",mode="xml",stringsAsFactors=TRUE,keep_flags=TRUE)
-bt2<-get_eurostat_data("agr_r_milkpr",keep_flags=TRUE,stringsAsFactors=FALSE)
-dt4<-get_eurostat_data("agr_r_milkpr",date_filter=2008,keep_flags=TRUE,stringsAsFactors=FALSE)
+teszt_id1<-"agr_r_milkpr"
+rt3<-get_eurostat_raw(teszt_id1,mode="xml",stringsAsFactors=TRUE,keep_flags=TRUE)
+bt2<-get_eurostat_data(teszt_id1,keep_flags=TRUE,stringsAsFactors=FALSE)
+dt4<-get_eurostat_data(teszt_id1,date_filter=2008,keep_flags=TRUE,stringsAsFactors=FALSE)
 if (!is.null(bt2)&!is.null(dt4)){
   test_that("test of the get_eurostat_raw/bulk/data function", {
-    expect_true(all.equal(bt2[time==2008,],dt4,check.attributes=FALSE))
+    expect_true(all.equal(bt2[time==2008,],dt4,check.attributes=FALSE,ignore.row.order=TRUE,ignore.col.order=TRUE))
   })
 }
 id<-"irt_h_eurcoe_d"
@@ -117,19 +130,20 @@ if (!is.null(bt3)&!is.null(bt4)){
   })
 }
 
+teszt_id2<-"avia_par_me"
 context("test of filtering in the get_eurostat_data function")
 test_that("test of filtering in the get_eurostat_data function", {
-  expect_message(dt5<-get_eurostat_data("agr_r_milkpr",filters="2018",stringsAsFactors=FALSE)) # date_filter value used for filters incorrectly => whole dataset downloaded
-  expect_message(dt6<-get_eurostat_data("agr_r_milkpr",date_filter=22020))
-  expect_message(dt7<-get_eurostat_data("agr_r_milkpr",date_filter="<2006<",cache_dir=tempdir()))
+  expect_message(dt5<-get_eurostat_data(teszt_id1,filters="2018",stringsAsFactors=FALSE)) # date_filter value used for filters incorrectly => whole dataset downloaded
+  expect_message(dt6<-get_eurostat_data(teszt_id1,date_filter=22020))
+  expect_message(dt7<-get_eurostat_data(teszt_id1,date_filter="<2006<",cache_dir=tempdir()))
   expect_true(identical(dt6,dt7))
-  expect_message(dt8<-get_eurostat_data("avia_par_me",filters="HU",date_filter="2017-03",select_freq="Q",label=TRUE))
+  expect_message(dt8<-get_eurostat_data(teszt_id2,filters="HU",date_filter="2017-03",select_freq="Q",label=TRUE))
   expect_false(any(sapply(dt5,is.factor)))
   expect_true(any(sapply(dt6,is.factor)))
   expect_true(any(sapply(dt7,is.factor)))
   expect_true(is.null(dt8))
 })
-dt9<-get_eurostat_data("agr_r_milkpr",filters="2018",cflags=TRUE)
+dt9<-get_eurostat_data(teszt_id1,filters="2018",cflags=TRUE)
 if (!is.null(dt9)&is.data.frame(dt9)){
   test_that("test filtering in the get_eurostat_data function", {
     expect_equal(nrow(dt9),as.numeric(xml_toc$values[xml_toc$code=="agr_r_milkpr"]))
@@ -138,10 +152,10 @@ if (!is.null(dt9)&is.data.frame(dt9)){
 dsd1<-get_eurostat_dsd("agr_r_milkpr")
 dsd2<-get_eurostat_dsd("avia_par_me")
 if (!is.null(dsd1)&is.data.frame(dsd1)){
-  dt10<-get_eurostat_data("agr_r_milkpr",filters="AT")
+  dt10<-get_eurostat_data(teszt_id1,filters="AT")
   nc10<-ncol(dt10)
   nr10<-nrow(dt10)
-  dt11<-get_eurostat_data("agr_r_milkpr",filters="AT",exact_match=FALSE,keep_flags=TRUE)
+  dt11<-get_eurostat_data(teszt_id1,filters="AT",exact_match=FALSE,keep_flags=TRUE)
   nc11<-ncol(dt11)
   nr11<-nrow(dt11)
   if (!is.null(dt10)&!is.null(dt11)&is.data.frame(dt10)&is.data.frame(dt11)){
@@ -150,32 +164,32 @@ if (!is.null(dsd1)&is.data.frame(dsd1)){
       expect_true(nr10<nr11)
     })
   }
-  nr12<-nrow(get_eurostat_data("agr_r_milkpr",date_filter=2016))
-  nr13<-nrow(get_eurostat_data("agr_r_milkpr",date_filter="2016",keep_flags=TRUE))
+  nr12<-nrow(get_eurostat_data(teszt_id1,date_filter=2016))
+  nr13<-nrow(get_eurostat_data(teszt_id1,date_filter="2016",keep_flags=TRUE))
   if (!is.null(nr12)&!is.null(nr13)){
     test_that("test filtering in the get_eurostat_data function", {
       expect_equal(nr12,nr13)
     })
   }
-  nr14<-nrow(get_eurostat_data("agr_r_milkpr",filters="AT",exact_match=FALSE,keep_flags=TRUE,ignore.case=TRUE))
+  nr14<-nrow(get_eurostat_data(teszt_id1,filters="AT",exact_match=FALSE,keep_flags=TRUE,ignore.case=TRUE))
   if (!is.null(nr14)&!is.null(dt10)){
     test_that("test filtering in the get_eurostat_data function", {
       expect_true(nr14>nr10)
     })
   } 
-  nr15<-nrow(get_eurostat_data("agr_r_milkpr",filters="BE$",date_filter="2006-02:2008-06-05",label=TRUE))
+  nr15<-nrow(get_eurostat_data(teszt_id1,filters="BE$",date_filter="2006-02:2008-06-05",label=TRUE))
   if (!is.null(nr15)){
     test_that("test filtering in the get_eurostat_data function", {
       expect_equal(nr15,2)
     })
   }
-  nr16<-nrow(get_eurostat_data("agr_r_milkpr",filters="BE",date_filter="<2008",cflags=TRUE))
+  nr16<-nrow(get_eurostat_data(teszt_id1,filters="BE",date_filter="<2008",cflags=TRUE))
   if (!is.null(nr16)){
     test_that("test filtering in the get_eurostat_data function", {
       expect_equal(nr16,11)
     })
   }  
-  nr17<-nrow(get_eurostat_data("avia_par_me",filters="Q...ME_LYPG_HU_LHBP+ME_LYTV_UA_UKKK",date_filter=c("2017-07-01"),select_freq="M",cflags=TRUE))
+  nr17<-nrow(get_eurostat_data(teszt_id2,filters="Q...ME_LYPG_HU_LHBP+ME_LYTV_UA_UKKK",date_filter=c("2017-07-01"),select_freq="M",cflags=TRUE))
   if (!is.null(nr17)){
     test_that("test filtering in the get_eurostat_data function", {
       expect_equal(nr17,48)
@@ -264,13 +278,13 @@ if (grepl("\\.amzn|-aws",Sys.info()['release'])) {
   dsd1<-get_eurostat_dsd("agr_r_milkpr")
   dsd2<-get_eurostat_dsd("avia_par_me")
   if (!is.null(dsd1)&is.data.frame(dsd1)){
-    nr11<-nrow(get_eurostat_data("agr_r_milkpr",filters="^BE$",date_filter=c(2002,"2008",2015:2017)))
+    nr11<-nrow(get_eurostat_data(teszt_id1,filters="^BE$",date_filter=c(2002,"2008",2015:2017)))
     if (!is.null(nr11)){
       test_that("test filtering in the get_eurostat_data function", {
         expect_true(nr11<=5)
       })
     }
-    nr12<-nrow(get_eurostat_data("agr_r_milkpr",filters="BE",date_filter=c(2008,"2002",2015:2017)))
+    nr12<-nrow(get_eurostat_data(teszt_id1,filters="BE",date_filter=c(2008,"2002",2015:2017)))
     if (!is.null(nr11)&!is.null(nr12)){
       test_that("test filtering in the get_eurostat_data function", {
         expect_equal(nr11,nr12)
@@ -278,21 +292,21 @@ if (grepl("\\.amzn|-aws",Sys.info()['release'])) {
     }
   }
   if (!is.null(dsd2)){
-    nr13<-nrow(get_eurostat_data("avia_par_me",filters="BE$",exact_match=FALSE,date_filter=c(2016,"2017-03","2017-05"),select_freq="A",label=TRUE,cflags=TRUE,verbose=FALSE))
+    nr13<-nrow(get_eurostat_data(teszt_id2,filters="BE$",exact_match=FALSE,date_filter=c(2016,"2017-03","2017-05"),select_freq="A",label=TRUE,cflags=TRUE,verbose=FALSE))
     if (!is.null(nr13)){
       test_that("test filtering in the get_eurostat_data function", {
         expect_equal(nr13,24)
       })
     }
-    nr14<-nrow(get_eurostat_data("avia_par_me",date_filter=c(2016,"2017-03","2017-05","2017-07-01"),select_freq="Q",cflags=TRUE))
+    nr14<-nrow(get_eurostat_data(teszt_id2,date_filter=c(2016,"2017-03","2017-05","2017-07-01"),select_freq="Q",cflags=TRUE))
     if (!is.null(nr14)){
       test_that("test filtering in the get_eurostat_data function", {
         expect_equal(nr14,1152)
       })
     }
-    dt5<-get_eurostat_data("avia_par_me",filters="Q...ME_LYPG_HU_LHBP+ME_LYTV_UA_UKKK",date_filter=c("2016-08","2017-07-01"),select_freq="M")
-    dt6<-get_eurostat_data("avia_par_me",filters=c("HU","Quarterly","Monthly"),date_filter=c("2016-08","2017-07-01"),stringsAsFactors=FALSE,label=TRUE)
-    dt7<-get_eurostat_data("avia_par_me",filters=c("ZHULIANY","BUDAPEST","Quarterly","Monthly"),exact_match=FALSE,date_filter=c("2016-08","2017-07-01"),name=TRUE)
+    dt5<-get_eurostat_data(teszt_id2,filters="Q...ME_LYPG_HU_LHBP+ME_LYTV_UA_UKKK",date_filter=c("2016-08","2017-07-01"),select_freq="M")
+    dt6<-get_eurostat_data(teszt_id2,filters=c("HU","Quarterly","Monthly"),date_filter=c("2016-08","2017-07-01"),stringsAsFactors=FALSE,label=TRUE)
+    dt7<-get_eurostat_data(teszt_id2,filters=c("ZHULIANY","BUDAPEST","Quarterly","Monthly"),exact_match=FALSE,date_filter=c("2016-08","2017-07-01"),name=TRUE)
     if (!is.null(dt5)&!is.null(dt7)){ #&!is.null(dt6)
       test_that("test filtering in the get_eurostat_data function", {
         expect_equal(dt5,dt7)
@@ -301,29 +315,32 @@ if (grepl("\\.amzn|-aws",Sys.info()['release'])) {
         expect_true(any(sapply(dt7,is.factor)))
       })
     }
-    dt8<-get_eurostat_data("avia_par_me",filters="BE$",date_filter=c("2017-03",2016,"2017-07-01",2012:2014),select_freq="Q",label=TRUE,verbose=FALSE,name=FALSE)
+    dt8<-get_eurostat_data(teszt_id2,filters="BE$",date_filter=c("2017-03",2016,"2017-07-01",2012:2014),select_freq="Q",label=TRUE,verbose=FALSE,name=FALSE)
     if (!is.null(dt8)){
       expect_true(nrow(dt8)<=5040)
       expect_true(ncol(dt8)<=5)
     }
-   dt9<-get_eurostat_data("avia_par_ee",select_freq="Q")
-   dt10<-get_eurostat_data("avia_par_ee",select_freq="Q")
+   teszt_id3<-"avia_par_ee"
+   dt9<-get_eurostat_data(teszt_id3,select_freq="Q")
+   dt10<-get_eurostat_data(teszt_id3,select_freq="Q")
    if (!is.null(dt9)&!is.null(dt10)){
      expect_true(identical(dt9,dt10))
    }
   } 
+  teszt_id4<-"avia_par_is"
   dsd3<-get_eurostat_dsd("avia_par_is")
   if (!is.null(dsd3)&is.data.frame(dsd3)){
-   nr16<-nrow(get_eurostat_data("avia_par_is",filters="Monthly",exact_match=FALSE,date_filter=c("<2018-07-01"),select_freq="A",label=TRUE,name=FALSE))
+   nr16<-nrow(get_eurostat_data(teszt_id4,filters="Monthly",exact_match=FALSE,date_filter=c("<2018-07-01"),select_freq="A",label=TRUE,name=FALSE))
    if (!is.null(nr16)){
      test_that("test filtering in the get_eurostat_data function", {
        expect_equal(nr16,4374)
      })
    }
   }
+  teszt_id5<-"bop_its6_det"
   dsd4<-get_eurostat_dsd("bop_its6_det")
   if (!is.null(dsd4)&is.data.frame(dsd4)){
-    nr17<-nrow(get_eurostat_data("bop_its6_det",filters=list(bop_item="SC",currency="MIO_EUR",partner="EXT_EU28",geo=c("EU28","HU"),time="2010:2017",stk_flow="BAL"),date_filter="2010:2012",select_freq="A",label=TRUE,name=FALSE))
+    nr17<-nrow(get_eurostat_data(teszt_id5,filters=list(bop_item="SC",currency="MIO_EUR",partner="EXT_EU28",geo=c("EU28","HU"),time="2010:2017",stk_flow="BAL"),date_filter="2010:2012",select_freq="A",label=TRUE,name=FALSE))
     if (!is.null(nr17)){
       test_that("test filtering in the get_eurostat_data function", {
         expect_equal(nr17,6)
@@ -445,31 +462,31 @@ if (grepl("\\.amzn|-aws",Sys.info()['release'])) {
   
   context("additional tests of the get/put_eurostat_cache function")
   clean_restatapi_cache()
-  id<-"ei_bsfs_q"
+  teszt_id6<-"ei_bsfs_q"
   xml_toc<-get_eurostat_toc(verbose=TRUE)
-  udate2<-xml_toc$lastUpdate[xml_toc$code==id]
+  udate2<-xml_toc$lastUpdate[xml_toc$code== teszt_id6]
   udate<-format(Sys.Date(),"%Y.%m.%d")
-  nm<-paste0("r_",id,"-",udate)
-  rt1<-system.time(raw1<-get_eurostat_raw(id,"xml",keep_flags=TRUE,stringsAsFactors=TRUE,verbose=TRUE))[3]
-  rt2<-system.time(raw2<-get_eurostat_raw(id,"xml",cache_dir=tempdir(),verbose=TRUE))[3]
-  bt1<-system.time(bulk1<-get_eurostat_bulk(id,stringsAsFactors=FALSE,verbose=TRUE))[3]
+  nm<-paste0("r_", teszt_id6,"-",udate)
+  rt1<-system.time(raw1<-get_eurostat_raw(teszt_id6,"xml",keep_flags=TRUE,stringsAsFactors=TRUE,verbose=TRUE))[3]
+  rt2<-system.time(raw2<-get_eurostat_raw(teszt_id6,"xml",cache_dir=tempdir(),verbose=TRUE))[3]
+  bt1<-system.time(bulk1<-get_eurostat_bulk(teszt_id6,stringsAsFactors=FALSE,verbose=TRUE))[3]
   nrb1<-nrow(bulk1)
   ncb1<-ncol(bulk1)
   cnb1<-colnames(bulk1)
-  rt3<-system.time(raw3<-get_eurostat_raw(id,"xml",verbose=TRUE))[3]
-  dt1<-system.time(estat_data1<-get_eurostat_data(id,keep_flags=TRUE,verbose=TRUE))[3]
-  dt2<-system.time(estat_data2<-get_eurostat_data(id,stringsAsFactors=FALSE,verbose=TRUE))[3]
+  rt3<-system.time(raw3<-get_eurostat_raw(teszt_id6,"xml",verbose=TRUE))[3]
+  dt1<-system.time(estat_data1<-get_eurostat_data(teszt_id6,keep_flags=TRUE,verbose=TRUE))[3]
+  dt2<-system.time(estat_data2<-get_eurostat_data(teszt_id6,stringsAsFactors=FALSE,verbose=TRUE))[3]
   kc<-colnames(bulk1)[1:(ncol(bulk1)-1)]
   data.table::setorderv(bulk1,kc)
   data.table::setorderv(estat_data2,kc)
   nrd2<-nrow(estat_data2)
   ncd2<-ncol(estat_data2)
   cnd2<-colnames(estat_data2)
-  dt3<-system.time(estat_data3<-get_eurostat_data(id,keep_flags=TRUE,verbose=TRUE))[3]
-  id<-"avia_par_mk"
-  suppressWarnings(dt4<-system.time(estat_data4<-get_eurostat_data(id,stringsAsFactors=FALSE))[3])
-  rt4<-system.time(raw4<-get_eurostat_raw(id,"xml",keep_flags=TRUE))[3]
-  suppressWarnings(bt2<-system.time(bulk2<-get_eurostat_bulk(id,keep_flags=TRUE,verbose=TRUE))[3])
+  dt3<-system.time(estat_data3<-get_eurostat_data(teszt_id6,keep_flags=TRUE,verbose=TRUE))[3]
+  teszt_id7<-"avia_par_mk"
+  suppressWarnings(dt4<-system.time(estat_data4<-get_eurostat_data(teszt_id7,stringsAsFactors=FALSE))[3])
+  rt4<-system.time(raw4<-get_eurostat_raw(teszt_id7,"xml",keep_flags=TRUE))[3]
+  suppressWarnings(bt2<-system.time(bulk2<-get_eurostat_bulk(teszt_id7,keep_flags=TRUE,verbose=TRUE))[3])
   if (!is.null(raw1)&is.data.frame(raw1)&!is.null(raw2)&is.data.frame(raw2)&!is.null(raw3)&is.data.frame(raw3)&!is.null(raw4)&is.data.frame(raw4)&!is.null(bulk1)&is.data.frame(bulk1)&!is.null(bulk2)&is.data.frame(bulk2)&!is.null(estat_data1)&is.data.frame(estat_data1)&!is.null(estat_data2)&is.data.frame(estat_data2)&!is.null(estat_data3)&is.data.frame(estat_data3)&!is.null(estat_data4)&is.data.frame(estat_data4)){
     test_that("test of the get/put_eurostat_cache function", {
       expect_true(exists(paste0(nm,"-1"),envir=.restatapi_env))
